"use strict";
this.name	     = "Engine_Sounds_FG";
this.version     = "1.1";    // Date 2015-12-06
this.description = "Creates an engine sound that follows the speed of the your ship and additional sounds when maneuvering or using the torus drive.";
this.author	     = "Fritz G.";   
this.copyright	 = "© 2015 Fritz G.";
this.licence	 = "CC BY-NC-SA 4.0";

//--------------------------------------------------------------------------------------------------------
// Credits:
// This OXP is based on ideas developed in http://oolite.aegidian.org/bb/viewtopic.php?f=2&t=17920
// and on a demonstration OXP by Norby. An updated version of Norby's OXP is available under 
// http://wiki.alioth.net/index.php/Engine_Sound. The sound files were taken from BGS OXP by Pagroove, 
// PhantorGorth, Thargoid, Tricky & Svengali. It is available under http://wiki.alioth.net/index.php/BGS.
// 
// Differences to Engine Sounds OXP by Norby:
// This OXP uses the same 25 sound files as Engine Sounds but choses them in relation speed/maxSpeed. 
// Norby's OXP uses the absolute value for speed for file selection, so slower ships will sound 
// differently at top speed than faster ships. With this OXP, all ships sound the same but can
// use the full "spectrum" of the available sound files.
// Further difference are that this OXP has a "standby sound" when speed is zero and the engine sound
// keeps running during torus driving. Finally, this OXP creates sounds on starting and stopping the 
// torus drive. The files used are taken from BGS OXP, where they are used for acceleration and deceleration. 
// This can be deactivated in the configuration section, where you can also adjust the volume levels.   
//
// Differences to the engine sounds in BGS OXP and compatibility issues:
// BGS uses a constant ambient engine sound when speed > 0 (using the sound file that is played at 25 
// different speeds in this and Norby's OXP). So, in BGS, you can't hear speed differences, and this 
// actually started the discussion mentioned above. BGS uses two additional sound files to create
// accelerating and deceleration noises - the files are used in this OXP for the torus drive.
// BGS and this OXP can be used together. Because of a combined "switch" for radio chatter and ambient 
// engine sound, using this OXP sadly also disables the chatter. This can only be fixed in BGS itself.
//--------------------------------------------------------------------------------------------------------
// configuration
this.$es_volume = 0.75;          // volume for engine and torus sounds 
this.$es_volumeStandby = 0.3;    // engine volume for speed = 0
this.$es_volumeManeuver = 0.5;   // volume for maneuvering sound
this.$es_torusSound = true;	     // change to false if you wish to disable the torus sounds
this.$es_maneuveringSound = 20;	 // sound file used for maneuvering sounds (1..25). Use 0 to disable.
//--------------------------------------------------------------------------------------------------------

// internal properties for engine sounds 
this.$es_soundsEngines = [];         // array for storing the 25 different sounds
this.$es_soundSourceEngines = null;  
this.$es_soundPlaying  = 1;          // (1 .. 25)

// internal properties for maneuvering sounds 
this.$es_soundSourceManeuver = null;   
this.$es_oldOrientation = null;            // stores the ship's orientation

// internal properties for torus sounds 
this.$es_soundsTorus = [];           // array for storing the 2 different sounds 
this.$es_soundSourceTorus = null;   
this.$es_torusOn = false;

this.$es_paused = false;            // game paused

// worldscript events
this.startUp = function() 
{
    // load and initialize engine sounds. 
	for(var i=1; i<26; i++) 
	{
		this.$es_soundsEngines[i] = Sound.load("bgs-m_ambi_engine1-"+i+".ogg");
	}
	this.$es_soundSourceEngines = new SoundSource;
	this.$es_soundSourceEngines.sound = this.$es_soundsEngines[1]; 
	this.$es_soundPlaying  = 1;   // actually the first sound played after launching will be somewhere in the middle range      
	this.$es_soundSourceEngines.loop = true;
	this.$es_soundSourceEngines.volume = 0;
	
	if (this.$es_torusSound)
	{	
		// initialize torus sounds
		this.$es_soundsTorus[1] = Sound.load("bgs-m_fx_engineup1.ogg");    
		this.$es_soundsTorus[2] = Sound.load("bgs-m_fx_enginedown1.ogg");
		this.$es_soundSourceTorus = new SoundSource;
		this.$es_soundSourceTorus.sound = $es_soundsTorus[1];
		this.$es_soundSourceTorus.loop = false;
		this.$es_soundSourceTorus.volume = this.$es_volume;
		this.$es_torusOn = false;
	}	

	if (this.$es_maneuveringSound > 0)
	{	
		// initialize maneuvering sounds 
		this.$es_soundSourceManeuver  = new SoundSource;
		this.$es_soundSourceManeuver.sound = Sound.load("bgs-m_ambi_engine1-"+this.$es_maneuveringSound+".ogg");
		this.$es_soundSourceManeuver.loop = true;
		this.$es_soundSourceManeuver.volume = this.$es_volumeManeuver; 
		// this.$es_soundSourceManeuver.play();    // play continuously, only volume will be changed
		this.$es_oldOrientation = new Quaternion();
	}	

   	// initialise timer. 0.25 is the minimal value so we need two timers 
	this.$es_timer1 =  new Timer(this, this.$es_changeSounds, 0, 0.25);        
	this.$es_timer2 =  new Timer(this, this.$es_changeSounds, 0.125, 0.25);
};

// BGS OXP ambience engine sound, accelerating and decelerating sounds must be deactivated. 
// Isn't perfect this way because BGS can't be configured to the needed detail, so 
// there will be more sounds deactivated than needed. 
this.startUpComplete = function() 
{
 	var b = worldScripts["BGS-M"];
	if(b) 
	{
		log ("Engine Sound: BGS-M detected");
		b.ambientSounds = false; // turns off BGS engine sound but  the chatter too
		b.fxSounds = false;      // turns off acellerating and decelerating engine sound effects 		
	}	
 };

// timer function called every 1/8 s
this.$es_changeSounds = function() 
{
	var p = player.ship;
	if( !p || !p.isValid || p.docked || this.$es_paused) 
	{
		this.$es_soundSourceEngines.volume = 0;   
		if (this.$es_maneuveringSound > 0) this.$es_soundSourceManeuver.stop();   
		return;
	}

	var pSpeed = p.speed/p.maxSpeed;  

	if (pSpeed >= 1) 
	{
		// note: speed > maxSpeed could mean injector speed too, but the injectors do have
		// their own sound in the core game, so we don't need to do anything in this case.
		// (injector speed is 7 x maxSpeed)
		if (this.$es_torusSound)
		{		
			if (p.torusEngaged && !this.$es_torusOn)
			{
				this.$es_torusOn = true;
				this.$es_soundSourceTorus.sound = this.$es_soundsTorus[1];
				this.$es_soundSourceTorus.play();
			}	
			else if (!p.torusEngaged && this.$es_torusOn) 
			{
				// torus drive stopped 
				this.$es_torusOn = false;
				this.$es_soundSourceTorus.sound = this.$es_soundsTorus[2];
				this.$es_soundSourceTorus.play();
			}	
		}		
		pSpeed = 1;    // keeps engines running at full speed (sound #25)
	}	

    if (this.$es_maneuveringSound > 0)
	{		
		// Make a maneuvering sound if the orientation has changed in any way. Because of inaccuracies 
		// when comparing float values, a threshold must be specified. 
		if (Math.abs(p.orientation.x - this.$es_oldOrientation.x ) > 0.0001 || 
			Math.abs(p.orientation.y - this.$es_oldOrientation.y ) > 0.0001 || 
			Math.abs(p.orientation.z - this.$es_oldOrientation.z ) > 0.0001)
		{	
			if (!this.$es_soundSourceManeuver.isPlaying) this.$es_soundSourceManeuver.play();
			this.$es_oldOrientation = p.orientation;	
		}
		else if (this.$es_soundSourceManeuver.isPlaying) 
		{	
			this.$es_soundSourceManeuver.stop();
		}
	}
	
    // To get a standby sound we use sound #1 für speed 0 and sound #2 for speeds even slightly higher than 0. 
	if (pSpeed === 0) 
	{
		var sound = 1;
		var vol = this.$es_volumeStandby;
	} 
	else 
	{
		var sound = Math.ceil(pSpeed*24)+1; // [2..25]
		var vol = this.$es_volume;
    }	
	
	if(sound != this.$es_soundPlaying) 
	{
		this.$es_soundSourceEngines.sound = this.$es_soundsEngines[sound];
		this.$es_soundSourceEngines.volume = vol;
		this.$es_soundSourceEngines.play();
		this.$es_soundPlaying = sound;
	}
};

this.gamePaused = function() 
{
	this.$es_soundSourceEngines.stop();   
    if (this.$es_maneuveringSound > 0) this.$es_soundSourceManeuver.stop();   
	this.$es_paused = true;
};

this.gameResumed = function() 
{
	this.$es_soundSourceEngines.play();   
	// starting play is not needed for maneuvering sound
	this.$es_paused = false;
};
